'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isEmpty = exports.isLoaded = exports.snapshotToJS = exports.customToJS = exports.dataToJS = exports.pathToJS = exports.toJS = exports.fixPath = undefined;

var _map2 = require('lodash/map');

var _map3 = _interopRequireDefault(_map2);

var _size2 = require('lodash/size');

var _size3 = _interopRequireDefault(_size2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @description Fix path by adding "/" to path if needed
 * @param {String} path - Path string to fix
 * @return {String} path - Fixed path
 */
var fixPath = exports.fixPath = function fixPath(path) {
  return (path.substring(0, 1) === '/' ? '' : '/') + path;
};

/**
 * @description Convert Immutable Map to a Javascript object
 * @param {Object} data - Immutable Map to be converted to JS object
 * @return {Object} data - Javascript version of Immutable Map
 */
var toJS = exports.toJS = function toJS(data) {
  if (data && data.toJS) {
    return data.toJS();
  }

  return data;
};

/**
 * @description Convert parameter from Immutable Map to a Javascript object
 * @param {Object} data - Immutable Map to be converted to JS object
 * @return {Object} data - Javascript version of Immutable Map
 */
var pathToJS = exports.pathToJS = function pathToJS(data, path, notSetValue) {
  if (!data) {
    return notSetValue;
  }

  var pathArr = fixPath(path).split(/\//).slice(1);

  if (data.getIn) {
    return toJS(data.getIn(pathArr, notSetValue));
  }

  return data;
};

/**
 * @description Convert parameter under "data" path of Immutable Map to a Javascript object
 * @param {Object} data - Immutable Map to be converted to JS object
 * @param {String} path - Path of parameter to load
 * @param {Object|String|Boolean} notSetValue - Value to return if value is not found
 * @return {Object} data - Javascript version of Immutable Map
 */
var dataToJS = exports.dataToJS = function dataToJS(data, path, notSetValue) {
  if (!data) {
    return notSetValue;
  }

  var dataPath = '/data' + fixPath(path);

  var pathArr = dataPath.split(/\//).slice(1);

  if (data.getIn) {
    return toJS(data.getIn(pathArr, notSetValue));
  }

  return data;
};

/**
 * @description Load custom object from within store
 * @param {Object} data - Immutable Map from store to be converted to JS object
 * @param {String} path - Path of parameter to load
 * @param {String} customPath - Part of store from which to load
 * @param {Object|String|Boolean} notSetValue - Value to return if value is not found
 * @return {Object} data - Javascript version of custom path within Immutable Map
 */
var customToJS = exports.customToJS = function customToJS(data, path, custom, notSetValue) {
  if (!(data && data.getIn)) {
    return notSetValue;
  }

  var customPath = '/' + custom + fixPath(path);

  var pathArr = customPath.split(/\//).slice(1);

  if (data.getIn) {
    return toJS(data.getIn(pathArr, notSetValue));
  }

  return data;
};

/**
 * @description Convert Immutable Map to a Javascript object
 * @param {Object} snapshot - Snapshot from store
 * @param {String} path - Path of snapshot to load
 * @return {Object} notSetValue - Value to return if snapshot is not found
 */
var snapshotToJS = exports.snapshotToJS = function snapshotToJS(snapshot, path, notSetValue) {
  if (!snapshot) {
    return notSetValue;
  }

  var snapshotPath = '/snapshot' + fixPath(path);

  var pathArr = snapshotPath.split(/\//).slice(1);

  if (snapshot.getIn) {
    return toJS(snapshot.getIn(pathArr, notSetValue));
  }

  return snapshot;
};

/**
 * @description Detect whether items are loaded yet or not
 * @param {Object} item - Item to check loaded status of. A comma seperated list is also acceptable.
 * @return {Boolean} isLoaded - Whether or not item is loaded
 */
var isLoaded = exports.isLoaded = function isLoaded() {
  if (!arguments || !arguments.length) {
    return true;
  }

  return (0, _map3.default)(arguments, function (a) {
    return a !== undefined;
  }).reduce(function (a, b) {
    return a && b;
  });
};

/**
 * @description Detect whether items are empty or not
 * @param {Object} item - Item to check loaded status of. A comma seperated list is also acceptable.
 * @return {Boolean} isEmpty - Whether or not item is empty
 */
var isEmpty = exports.isEmpty = function isEmpty(data) {
  return !(data && (0, _size3.default)(data));
};

exports.default = {
  toJS: toJS,
  pathToJS: pathToJS,
  dataToJS: dataToJS,
  snapshotToJS: snapshotToJS,
  customToJS: customToJS,
  isLoaded: isLoaded,
  isEmpty: isEmpty
};